//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;
#[cfg(feature = "objc2-user-notifications")]
use objc2_user_notifications::*;

use crate::*;

extern_category!(
    /// Category "UIKitAdditions" on [`UNNotificationResponse`].
    #[doc(alias = "UIKitAdditions")]
    pub unsafe trait UNNotificationResponseUIKitAdditions {
        #[cfg(all(feature = "UIResponder", feature = "UIScene"))]
        #[method_id(@__retain_semantics Other targetScene)]
        unsafe fn targetScene(&self, mtm: MainThreadMarker) -> Option<Retained<UIScene>>;
    }

    #[cfg(feature = "objc2-user-notifications")]
    unsafe impl UNNotificationResponseUIKitAdditions for UNNotificationResponse {}
);
