//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;

use crate::*;

// NS_OPTIONS
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct UIDataDetectorTypes(pub NSUInteger);
bitflags::bitflags! {
    impl UIDataDetectorTypes: NSUInteger {
        const UIDataDetectorTypePhoneNumber = 1<<0;
        const UIDataDetectorTypeLink = 1<<1;
        const UIDataDetectorTypeAddress = 1<<2;
        const UIDataDetectorTypeCalendarEvent = 1<<3;
        const UIDataDetectorTypeShipmentTrackingNumber = 1<<4;
        const UIDataDetectorTypeFlightNumber = 1<<5;
        const UIDataDetectorTypeLookupSuggestion = 1<<6;
        const UIDataDetectorTypeMoney = 1<<7;
        const UIDataDetectorTypePhysicalValue = 1<<8;
        const UIDataDetectorTypeNone = 0;
        const UIDataDetectorTypeAll = NSUIntegerMax as _;
    }
}

unsafe impl Encode for UIDataDetectorTypes {
    const ENCODING: Encoding = NSUInteger::ENCODING;
}

unsafe impl RefEncode for UIDataDetectorTypes {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}
