//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_protocol!(
    pub unsafe trait UIHoverEffect: NSCopying + NSObjectProtocol + IsMainThreadOnly {}

    unsafe impl ProtocolType for dyn UIHoverEffect {}
);

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIHoverHighlightEffect;

    unsafe impl ClassType for UIHoverHighlightEffect {
        type Super = NSObject;
        type Mutability = MainThreadOnly;
    }
);

unsafe impl NSCopying for UIHoverHighlightEffect {}

unsafe impl NSObjectProtocol for UIHoverHighlightEffect {}

unsafe impl UIHoverEffect for UIHoverHighlightEffect {}

extern_methods!(
    unsafe impl UIHoverHighlightEffect {
        #[method_id(@__retain_semantics Other effect)]
        pub unsafe fn effect(mtm: MainThreadMarker) -> Retained<Self>;

        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;
    }
);

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIHoverLiftEffect;

    unsafe impl ClassType for UIHoverLiftEffect {
        type Super = NSObject;
        type Mutability = MainThreadOnly;
    }
);

unsafe impl NSCopying for UIHoverLiftEffect {}

unsafe impl NSObjectProtocol for UIHoverLiftEffect {}

unsafe impl UIHoverEffect for UIHoverLiftEffect {}

extern_methods!(
    unsafe impl UIHoverLiftEffect {
        #[method_id(@__retain_semantics Other effect)]
        pub unsafe fn effect(mtm: MainThreadMarker) -> Retained<Self>;

        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;
    }
);

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIHoverAutomaticEffect;

    unsafe impl ClassType for UIHoverAutomaticEffect {
        type Super = NSObject;
        type Mutability = MainThreadOnly;
    }
);

unsafe impl NSCopying for UIHoverAutomaticEffect {}

unsafe impl NSObjectProtocol for UIHoverAutomaticEffect {}

unsafe impl UIHoverEffect for UIHoverAutomaticEffect {}

extern_methods!(
    unsafe impl UIHoverAutomaticEffect {
        #[method_id(@__retain_semantics Other effect)]
        pub unsafe fn effect(mtm: MainThreadMarker) -> Retained<Self>;

        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;
    }
);
