//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIKey;

    unsafe impl ClassType for UIKey {
        type Super = NSObject;
        type Mutability = MainThreadOnly;
    }
);

unsafe impl NSCoding for UIKey {}

unsafe impl NSCopying for UIKey {}

unsafe impl NSObjectProtocol for UIKey {}

extern_methods!(
    unsafe impl UIKey {
        #[method_id(@__retain_semantics Other characters)]
        pub unsafe fn characters(&self) -> Retained<NSString>;

        #[method_id(@__retain_semantics Other charactersIgnoringModifiers)]
        pub unsafe fn charactersIgnoringModifiers(&self) -> Retained<NSString>;

        #[cfg(feature = "UICommand")]
        #[method(modifierFlags)]
        pub unsafe fn modifierFlags(&self) -> UIKeyModifierFlags;

        #[cfg(feature = "UIKeyConstants")]
        #[method(keyCode)]
        pub unsafe fn keyCode(&self) -> UIKeyboardHIDUsage;
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl UIKey {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;
    }
);
