//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

#[cfg(all(feature = "UIWindowSceneActivationConfiguration", feature = "block2"))]
pub type UIWindowSceneActivationInteractionConfigurationProvider = *mut block2::Block<
    dyn Fn(
        NonNull<UIWindowSceneActivationInteraction>,
        CGPoint,
    ) -> *mut UIWindowSceneActivationConfiguration,
>;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIWindowSceneActivationInteraction;

    unsafe impl ClassType for UIWindowSceneActivationInteraction {
        type Super = NSObject;
        type Mutability = MainThreadOnly;
    }
);

unsafe impl NSObjectProtocol for UIWindowSceneActivationInteraction {}

#[cfg(feature = "UIInteraction")]
unsafe impl UIInteraction for UIWindowSceneActivationInteraction {}

extern_methods!(
    unsafe impl UIWindowSceneActivationInteraction {
        #[cfg(all(feature = "UIWindowSceneActivationConfiguration", feature = "block2"))]
        #[method_id(@__retain_semantics Init initWithConfigurationProvider:errorHandler:)]
        pub unsafe fn initWithConfigurationProvider_errorHandler(
            this: Allocated<Self>,
            configuration_provider: UIWindowSceneActivationInteractionConfigurationProvider,
            error_handler: &block2::Block<dyn Fn(NonNull<NSError>)>,
        ) -> Retained<Self>;

        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;
    }
);
