//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIBarButtonItemStateAppearance;

    unsafe impl ClassType for UIBarButtonItemStateAppearance {
        type Super = NSObject;
        type Mutability = MainThreadOnly;
    }
);

unsafe impl NSObjectProtocol for UIBarButtonItemStateAppearance {}

extern_methods!(
    unsafe impl UIBarButtonItemStateAppearance {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;

        #[method_id(@__retain_semantics Other titleTextAttributes)]
        pub unsafe fn titleTextAttributes(
            &self,
        ) -> Retained<NSDictionary<NSAttributedStringKey, AnyObject>>;

        #[method(setTitleTextAttributes:)]
        pub unsafe fn setTitleTextAttributes(
            &self,
            title_text_attributes: &NSDictionary<NSAttributedStringKey, AnyObject>,
        );

        #[cfg(feature = "UIGeometry")]
        #[method(titlePositionAdjustment)]
        pub unsafe fn titlePositionAdjustment(&self) -> UIOffset;

        #[cfg(feature = "UIGeometry")]
        #[method(setTitlePositionAdjustment:)]
        pub unsafe fn setTitlePositionAdjustment(&self, title_position_adjustment: UIOffset);

        #[cfg(feature = "UIImage")]
        #[method_id(@__retain_semantics Other backgroundImage)]
        pub unsafe fn backgroundImage(&self) -> Option<Retained<UIImage>>;

        #[cfg(feature = "UIImage")]
        #[method(setBackgroundImage:)]
        pub unsafe fn setBackgroundImage(&self, background_image: Option<&UIImage>);

        #[cfg(feature = "UIGeometry")]
        #[method(backgroundImagePositionAdjustment)]
        pub unsafe fn backgroundImagePositionAdjustment(&self) -> UIOffset;

        #[cfg(feature = "UIGeometry")]
        #[method(setBackgroundImagePositionAdjustment:)]
        pub unsafe fn setBackgroundImagePositionAdjustment(
            &self,
            background_image_position_adjustment: UIOffset,
        );
    }
);

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIBarButtonItemAppearance;

    unsafe impl ClassType for UIBarButtonItemAppearance {
        type Super = NSObject;
        type Mutability = MainThreadOnly;
    }
);

unsafe impl NSCoding for UIBarButtonItemAppearance {}

unsafe impl NSCopying for UIBarButtonItemAppearance {}

unsafe impl NSObjectProtocol for UIBarButtonItemAppearance {}

unsafe impl NSSecureCoding for UIBarButtonItemAppearance {}

extern_methods!(
    unsafe impl UIBarButtonItemAppearance {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[cfg(feature = "UIBarButtonItem")]
        #[method_id(@__retain_semantics Init initWithStyle:)]
        pub unsafe fn initWithStyle(
            this: Allocated<Self>,
            style: UIBarButtonItemStyle,
        ) -> Retained<Self>;

        #[method_id(@__retain_semantics Init initWithCoder:)]
        pub unsafe fn initWithCoder(this: Allocated<Self>, coder: &NSCoder) -> Retained<Self>;

        #[method_id(@__retain_semantics CopyOrMutCopy copy)]
        pub unsafe fn copy(&self) -> Retained<Self>;

        #[cfg(feature = "UIBarButtonItem")]
        #[method(configureWithDefaultForStyle:)]
        pub unsafe fn configureWithDefaultForStyle(&self, style: UIBarButtonItemStyle);

        #[method_id(@__retain_semantics Other normal)]
        pub unsafe fn normal(&self) -> Retained<UIBarButtonItemStateAppearance>;

        #[method_id(@__retain_semantics Other highlighted)]
        pub unsafe fn highlighted(&self) -> Retained<UIBarButtonItemStateAppearance>;

        #[method_id(@__retain_semantics Other disabled)]
        pub unsafe fn disabled(&self) -> Retained<UIBarButtonItemStateAppearance>;

        #[method_id(@__retain_semantics Other focused)]
        pub unsafe fn focused(&self) -> Retained<UIBarButtonItemStateAppearance>;
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl UIBarButtonItemAppearance {
        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;
    }
);
