//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

#[cfg(all(feature = "UIResponder", feature = "UIView", feature = "block2"))]
pub type UIContextualActionHandler = *mut block2::Block<
    dyn Fn(NonNull<UIContextualAction>, NonNull<UIView>, NonNull<block2::Block<dyn Fn(Bool)>>),
>;

// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct UIContextualActionStyle(pub NSInteger);
impl UIContextualActionStyle {
    #[doc(alias = "UIContextualActionStyleNormal")]
    pub const Normal: Self = Self(0);
    #[doc(alias = "UIContextualActionStyleDestructive")]
    pub const Destructive: Self = Self(1);
}

unsafe impl Encode for UIContextualActionStyle {
    const ENCODING: Encoding = NSInteger::ENCODING;
}

unsafe impl RefEncode for UIContextualActionStyle {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIContextualAction;

    unsafe impl ClassType for UIContextualAction {
        type Super = NSObject;
        type Mutability = MainThreadOnly;
    }
);

unsafe impl NSObjectProtocol for UIContextualAction {}

extern_methods!(
    unsafe impl UIContextualAction {
        #[cfg(all(feature = "UIResponder", feature = "UIView", feature = "block2"))]
        #[method_id(@__retain_semantics Other contextualActionWithStyle:title:handler:)]
        pub unsafe fn contextualActionWithStyle_title_handler(
            style: UIContextualActionStyle,
            title: Option<&NSString>,
            handler: UIContextualActionHandler,
            mtm: MainThreadMarker,
        ) -> Retained<Self>;

        #[method(style)]
        pub unsafe fn style(&self) -> UIContextualActionStyle;

        #[cfg(all(feature = "UIResponder", feature = "UIView", feature = "block2"))]
        #[method(handler)]
        pub unsafe fn handler(&self) -> UIContextualActionHandler;

        #[method_id(@__retain_semantics Other title)]
        pub unsafe fn title(&self) -> Option<Retained<NSString>>;

        #[method(setTitle:)]
        pub unsafe fn setTitle(&self, title: Option<&NSString>);

        #[cfg(feature = "UIColor")]
        #[method_id(@__retain_semantics Other backgroundColor)]
        pub unsafe fn backgroundColor(&self) -> Retained<UIColor>;

        #[cfg(feature = "UIColor")]
        #[method(setBackgroundColor:)]
        pub unsafe fn setBackgroundColor(&self, background_color: Option<&UIColor>);

        #[cfg(feature = "UIImage")]
        #[method_id(@__retain_semantics Other image)]
        pub unsafe fn image(&self) -> Option<Retained<UIImage>>;

        #[cfg(feature = "UIImage")]
        #[method(setImage:)]
        pub unsafe fn setImage(&self, image: Option<&UIImage>);
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl UIContextualAction {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;
    }
);
