//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct NSShadow;

    unsafe impl ClassType for NSShadow {
        type Super = NSObject;
        type Mutability = InteriorMutable;
    }
);

unsafe impl NSCoding for NSShadow {}

unsafe impl NSCopying for NSShadow {}

unsafe impl NSObjectProtocol for NSShadow {}

unsafe impl NSSecureCoding for NSShadow {}

extern_methods!(
    unsafe impl NSShadow {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics Init initWithCoder:)]
        pub unsafe fn initWithCoder(
            this: Allocated<Self>,
            coder: &NSCoder,
        ) -> Option<Retained<Self>>;

        #[method(shadowOffset)]
        pub unsafe fn shadowOffset(&self) -> CGSize;

        #[method(setShadowOffset:)]
        pub unsafe fn setShadowOffset(&self, shadow_offset: CGSize);

        #[method(shadowBlurRadius)]
        pub unsafe fn shadowBlurRadius(&self) -> CGFloat;

        #[method(setShadowBlurRadius:)]
        pub unsafe fn setShadowBlurRadius(&self, shadow_blur_radius: CGFloat);

        #[method_id(@__retain_semantics Other shadowColor)]
        pub unsafe fn shadowColor(&self) -> Option<Retained<AnyObject>>;

        #[method(setShadowColor:)]
        pub unsafe fn setShadowColor(&self, shadow_color: Option<&AnyObject>);
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl NSShadow {
        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new() -> Retained<Self>;
    }
);
