//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;

use crate::*;

// NS_ENUM
#[deprecated = "UITextItemInteraction has been replaced by text item methods on UITextViewDelegate"]
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct UITextItemInteraction(pub NSInteger);
impl UITextItemInteraction {
    #[deprecated = "UITextItemInteraction has been replaced by text item methods on UITextViewDelegate"]
    #[doc(alias = "UITextItemInteractionInvokeDefaultAction")]
    pub const InvokeDefaultAction: Self = Self(0);
    #[deprecated = "UITextItemInteraction has been replaced by text item methods on UITextViewDelegate"]
    #[doc(alias = "UITextItemInteractionPresentActions")]
    pub const PresentActions: Self = Self(1);
    #[deprecated = "UITextItemInteraction has been replaced by text item methods on UITextViewDelegate"]
    #[doc(alias = "UITextItemInteractionPreview")]
    pub const Preview: Self = Self(2);
}

unsafe impl Encode for UITextItemInteraction {
    const ENCODING: Encoding = NSInteger::ENCODING;
}

unsafe impl RefEncode for UITextItemInteraction {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}
