//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

// NS_TYPED_ENUM
pub type UIAccessibilityTraits = u64;

extern "C" {
    pub static UIAccessibilityTraitNone: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitButton: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitLink: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitHeader: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitSearchField: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitImage: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitSelected: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitPlaysSound: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitKeyboardKey: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitStaticText: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitSummaryElement: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitNotEnabled: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitUpdatesFrequently: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitStartsMediaSession: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitAdjustable: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitAllowsDirectInteraction: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitCausesPageTurn: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitTabBar: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitToggleButton: UIAccessibilityTraits;
}

extern "C" {
    pub static UIAccessibilityTraitSupportsZoom: UIAccessibilityTraits;
}

// NS_TYPED_ENUM
pub type UIAccessibilityNotifications = u32;

extern "C" {
    pub static UIAccessibilityScreenChangedNotification: UIAccessibilityNotifications;
}

extern "C" {
    pub static UIAccessibilityLayoutChangedNotification: UIAccessibilityNotifications;
}

extern "C" {
    pub static UIAccessibilityAnnouncementNotification: UIAccessibilityNotifications;
}

extern "C" {
    pub static UIAccessibilityPageScrolledNotification: UIAccessibilityNotifications;
}

extern "C" {
    pub static UIAccessibilityPauseAssistiveTechnologyNotification: UIAccessibilityNotifications;
}

extern "C" {
    pub static UIAccessibilityResumeAssistiveTechnologyNotification: UIAccessibilityNotifications;
}

extern "C" {
    pub static UIAccessibilityAnnouncementDidFinishNotification: &'static NSNotificationName;
}

extern "C" {
    pub static UIAccessibilityAnnouncementKeyStringValue: &'static NSString;
}

extern "C" {
    pub static UIAccessibilityAnnouncementKeyWasSuccessful: &'static NSString;
}

extern "C" {
    pub static UIAccessibilityElementFocusedNotification: &'static NSNotificationName;
}

extern "C" {
    pub static UIAccessibilityFocusedElementKey: &'static NSString;
}

extern "C" {
    pub static UIAccessibilityUnfocusedElementKey: &'static NSString;
}

extern "C" {
    pub static UIAccessibilityAssistiveTechnologyKey: &'static NSString;
}

// NS_TYPED_ENUM
pub type UIAccessibilityAssistiveTechnologyIdentifier = NSString;

extern "C" {
    pub static UIAccessibilityNotificationSwitchControlIdentifier:
        &'static UIAccessibilityAssistiveTechnologyIdentifier;
}

extern "C" {
    pub static UIAccessibilityNotificationVoiceOverIdentifier:
        &'static UIAccessibilityAssistiveTechnologyIdentifier;
}

// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct UIAccessibilityNavigationStyle(pub NSInteger);
impl UIAccessibilityNavigationStyle {
    #[doc(alias = "UIAccessibilityNavigationStyleAutomatic")]
    pub const Automatic: Self = Self(0);
    #[doc(alias = "UIAccessibilityNavigationStyleSeparate")]
    pub const Separate: Self = Self(1);
    #[doc(alias = "UIAccessibilityNavigationStyleCombined")]
    pub const Combined: Self = Self(2);
}

unsafe impl Encode for UIAccessibilityNavigationStyle {
    const ENCODING: Encoding = NSInteger::ENCODING;
}

unsafe impl RefEncode for UIAccessibilityNavigationStyle {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct UIAccessibilityContainerType(pub NSInteger);
impl UIAccessibilityContainerType {
    #[doc(alias = "UIAccessibilityContainerTypeNone")]
    pub const None: Self = Self(0);
    #[doc(alias = "UIAccessibilityContainerTypeDataTable")]
    pub const DataTable: Self = Self(1);
    #[doc(alias = "UIAccessibilityContainerTypeList")]
    pub const List: Self = Self(2);
    #[doc(alias = "UIAccessibilityContainerTypeLandmark")]
    pub const Landmark: Self = Self(3);
    #[doc(alias = "UIAccessibilityContainerTypeSemanticGroup")]
    pub const SemanticGroup: Self = Self(4);
}

unsafe impl Encode for UIAccessibilityContainerType {
    const ENCODING: Encoding = NSInteger::ENCODING;
}

unsafe impl RefEncode for UIAccessibilityContainerType {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

// NS_OPTIONS
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct UIAccessibilityDirectTouchOptions(pub NSUInteger);
bitflags::bitflags! {
    impl UIAccessibilityDirectTouchOptions: NSUInteger {
        const UIAccessibilityDirectTouchOptionNone = 0;
        const UIAccessibilityDirectTouchOptionSilentOnTouch = 1<<0;
        const UIAccessibilityDirectTouchOptionRequiresActivation = 1<<1;
    }
}

unsafe impl Encode for UIAccessibilityDirectTouchOptions {
    const ENCODING: Encoding = NSUInteger::ENCODING;
}

unsafe impl RefEncode for UIAccessibilityDirectTouchOptions {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

// NS_TYPED_EXTENSIBLE_ENUM
pub type UIAccessibilityTextualContext = NSString;

extern "C" {
    pub static UIAccessibilityTextualContextWordProcessing: &'static UIAccessibilityTextualContext;
}

extern "C" {
    pub static UIAccessibilityTextualContextNarrative: &'static UIAccessibilityTextualContext;
}

extern "C" {
    pub static UIAccessibilityTextualContextMessaging: &'static UIAccessibilityTextualContext;
}

extern "C" {
    pub static UIAccessibilityTextualContextSpreadsheet: &'static UIAccessibilityTextualContext;
}

extern "C" {
    pub static UIAccessibilityTextualContextFileSystem: &'static UIAccessibilityTextualContext;
}

extern "C" {
    pub static UIAccessibilityTextualContextSourceCode: &'static UIAccessibilityTextualContext;
}

extern "C" {
    pub static UIAccessibilityTextualContextConsole: &'static UIAccessibilityTextualContext;
}

// NS_TYPED_ENUM
pub type UIAccessibilityPriority = NSString;

extern "C" {
    pub static UIAccessibilityPriorityHigh: &'static UIAccessibilityPriority;
}

extern "C" {
    pub static UIAccessibilityPriorityDefault: &'static UIAccessibilityPriority;
}

extern "C" {
    pub static UIAccessibilityPriorityLow: &'static UIAccessibilityPriority;
}

extern "C" {
    pub static UIAccessibilitySpeechAttributePunctuation: &'static NSAttributedStringKey;
}

extern "C" {
    pub static UIAccessibilitySpeechAttributeLanguage: &'static NSAttributedStringKey;
}

extern "C" {
    pub static UIAccessibilitySpeechAttributePitch: &'static NSAttributedStringKey;
}

extern "C" {
    pub static UIAccessibilitySpeechAttributeQueueAnnouncement: &'static NSAttributedStringKey;
}

extern "C" {
    pub static UIAccessibilitySpeechAttributeAnnouncementPriority: &'static NSAttributedStringKey;
}

extern "C" {
    pub static UIAccessibilitySpeechAttributeIPANotation: &'static NSAttributedStringKey;
}

extern "C" {
    pub static UIAccessibilitySpeechAttributeSpellOut: &'static NSAttributedStringKey;
}

extern "C" {
    pub static UIAccessibilityTextAttributeHeadingLevel: &'static NSAttributedStringKey;
}

extern "C" {
    pub static UIAccessibilityTextAttributeCustom: &'static NSAttributedStringKey;
}

extern "C" {
    pub static UIAccessibilityTextAttributeContext: &'static NSAttributedStringKey;
}
