//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIDropInteraction;

    unsafe impl ClassType for UIDropInteraction {
        type Super = NSObject;
        type Mutability = MainThreadOnly;
    }
);

unsafe impl NSObjectProtocol for UIDropInteraction {}

#[cfg(feature = "UIInteraction")]
unsafe impl UIInteraction for UIDropInteraction {}

extern_methods!(
    unsafe impl UIDropInteraction {
        #[method_id(@__retain_semantics Init initWithDelegate:)]
        pub unsafe fn initWithDelegate(
            this: Allocated<Self>,
            delegate: &ProtocolObject<dyn UIDropInteractionDelegate>,
        ) -> Retained<Self>;

        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;

        #[method_id(@__retain_semantics Other delegate)]
        pub unsafe fn delegate(
            &self,
        ) -> Option<Retained<ProtocolObject<dyn UIDropInteractionDelegate>>>;

        #[method(allowsSimultaneousDropSessions)]
        pub unsafe fn allowsSimultaneousDropSessions(&self) -> bool;

        #[method(setAllowsSimultaneousDropSessions:)]
        pub unsafe fn setAllowsSimultaneousDropSessions(
            &self,
            allows_simultaneous_drop_sessions: bool,
        );
    }
);

// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct UIDropOperation(pub NSUInteger);
impl UIDropOperation {
    #[doc(alias = "UIDropOperationCancel")]
    pub const Cancel: Self = Self(0);
    #[doc(alias = "UIDropOperationForbidden")]
    pub const Forbidden: Self = Self(1);
    #[doc(alias = "UIDropOperationCopy")]
    pub const Copy: Self = Self(2);
    #[doc(alias = "UIDropOperationMove")]
    pub const Move: Self = Self(3);
}

unsafe impl Encode for UIDropOperation {
    const ENCODING: Encoding = NSUInteger::ENCODING;
}

unsafe impl RefEncode for UIDropOperation {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIDropProposal;

    unsafe impl ClassType for UIDropProposal {
        type Super = NSObject;
        type Mutability = MainThreadOnly;
    }
);

unsafe impl NSCopying for UIDropProposal {}

unsafe impl NSObjectProtocol for UIDropProposal {}

extern_methods!(
    unsafe impl UIDropProposal {
        #[method_id(@__retain_semantics Init initWithDropOperation:)]
        pub unsafe fn initWithDropOperation(
            this: Allocated<Self>,
            operation: UIDropOperation,
        ) -> Retained<Self>;

        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;

        #[method(operation)]
        pub unsafe fn operation(&self) -> UIDropOperation;

        #[method(isPrecise)]
        pub unsafe fn isPrecise(&self) -> bool;

        #[method(setPrecise:)]
        pub unsafe fn setPrecise(&self, precise: bool);

        #[method(prefersFullSizePreview)]
        pub unsafe fn prefersFullSizePreview(&self) -> bool;

        #[method(setPrefersFullSizePreview:)]
        pub unsafe fn setPrefersFullSizePreview(&self, prefers_full_size_preview: bool);
    }
);

extern_protocol!(
    pub unsafe trait UIDropInteractionDelegate: NSObjectProtocol + IsMainThreadOnly {
        #[cfg(feature = "UIDragSession")]
        #[optional]
        #[method(dropInteraction:canHandleSession:)]
        unsafe fn dropInteraction_canHandleSession(
            &self,
            interaction: &UIDropInteraction,
            session: &ProtocolObject<dyn UIDropSession>,
        ) -> bool;

        #[cfg(feature = "UIDragSession")]
        #[optional]
        #[method(dropInteraction:sessionDidEnter:)]
        unsafe fn dropInteraction_sessionDidEnter(
            &self,
            interaction: &UIDropInteraction,
            session: &ProtocolObject<dyn UIDropSession>,
        );

        #[cfg(feature = "UIDragSession")]
        #[optional]
        #[method_id(@__retain_semantics Other dropInteraction:sessionDidUpdate:)]
        unsafe fn dropInteraction_sessionDidUpdate(
            &self,
            interaction: &UIDropInteraction,
            session: &ProtocolObject<dyn UIDropSession>,
        ) -> Retained<UIDropProposal>;

        #[cfg(feature = "UIDragSession")]
        #[optional]
        #[method(dropInteraction:sessionDidExit:)]
        unsafe fn dropInteraction_sessionDidExit(
            &self,
            interaction: &UIDropInteraction,
            session: &ProtocolObject<dyn UIDropSession>,
        );

        #[cfg(feature = "UIDragSession")]
        #[optional]
        #[method(dropInteraction:performDrop:)]
        unsafe fn dropInteraction_performDrop(
            &self,
            interaction: &UIDropInteraction,
            session: &ProtocolObject<dyn UIDropSession>,
        );

        #[cfg(feature = "UIDragSession")]
        #[optional]
        #[method(dropInteraction:concludeDrop:)]
        unsafe fn dropInteraction_concludeDrop(
            &self,
            interaction: &UIDropInteraction,
            session: &ProtocolObject<dyn UIDropSession>,
        );

        #[cfg(feature = "UIDragSession")]
        #[optional]
        #[method(dropInteraction:sessionDidEnd:)]
        unsafe fn dropInteraction_sessionDidEnd(
            &self,
            interaction: &UIDropInteraction,
            session: &ProtocolObject<dyn UIDropSession>,
        );

        #[cfg(all(
            feature = "UIDragItem",
            feature = "UITargetedDragPreview",
            feature = "UITargetedPreview"
        ))]
        #[optional]
        #[method_id(@__retain_semantics Other dropInteraction:previewForDroppingItem:withDefault:)]
        unsafe fn dropInteraction_previewForDroppingItem_withDefault(
            &self,
            interaction: &UIDropInteraction,
            item: &UIDragItem,
            default_preview: &UITargetedDragPreview,
        ) -> Option<Retained<UITargetedDragPreview>>;

        #[cfg(all(feature = "UIDragInteraction", feature = "UIDragItem"))]
        #[optional]
        #[method(dropInteraction:item:willAnimateDropWithAnimator:)]
        unsafe fn dropInteraction_item_willAnimateDropWithAnimator(
            &self,
            interaction: &UIDropInteraction,
            item: &UIDragItem,
            animator: &ProtocolObject<dyn UIDragAnimating>,
        );
    }

    unsafe impl ProtocolType for dyn UIDropInteractionDelegate {}
);
