//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIImageAsset;

    unsafe impl ClassType for UIImageAsset {
        type Super = NSObject;
        type Mutability = InteriorMutable;
    }
);

unsafe impl Send for UIImageAsset {}

unsafe impl Sync for UIImageAsset {}

unsafe impl NSCoding for UIImageAsset {}

unsafe impl NSObjectProtocol for UIImageAsset {}

unsafe impl NSSecureCoding for UIImageAsset {}

extern_methods!(
    unsafe impl UIImageAsset {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics Init initWithCoder:)]
        pub unsafe fn initWithCoder(
            this: Allocated<Self>,
            coder: &NSCoder,
        ) -> Option<Retained<Self>>;

        #[cfg(all(feature = "UIImage", feature = "UIImageConfiguration"))]
        #[method_id(@__retain_semantics Other imageWithConfiguration:)]
        pub unsafe fn imageWithConfiguration(
            &self,
            configuration: &UIImageConfiguration,
        ) -> Retained<UIImage>;

        #[cfg(all(feature = "UIImage", feature = "UIImageConfiguration"))]
        #[method(registerImage:withConfiguration:)]
        pub unsafe fn registerImage_withConfiguration(
            &self,
            image: &UIImage,
            configuration: &UIImageConfiguration,
        );

        #[cfg(feature = "UIImageConfiguration")]
        #[method(unregisterImageWithConfiguration:)]
        pub unsafe fn unregisterImageWithConfiguration(&self, configuration: &UIImageConfiguration);

        #[cfg(all(feature = "UIImage", feature = "UITraitCollection"))]
        #[method_id(@__retain_semantics Other imageWithTraitCollection:)]
        pub unsafe fn imageWithTraitCollection(
            &self,
            trait_collection: &UITraitCollection,
        ) -> Retained<UIImage>;

        #[cfg(all(feature = "UIImage", feature = "UITraitCollection"))]
        #[method(registerImage:withTraitCollection:)]
        pub unsafe fn registerImage_withTraitCollection(
            &self,
            image: &UIImage,
            trait_collection: &UITraitCollection,
        );

        #[cfg(feature = "UITraitCollection")]
        #[method(unregisterImageWithTraitCollection:)]
        pub unsafe fn unregisterImageWithTraitCollection(
            &self,
            trait_collection: &UITraitCollection,
        );
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl UIImageAsset {
        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new() -> Retained<Self>;
    }
);
