//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;

use crate::*;

#[cfg(all(feature = "UIColor", feature = "block2"))]
pub type UIConfigurationColorTransformer =
    *mut block2::Block<dyn Fn(NonNull<UIColor>) -> NonNull<UIColor>>;

extern "C" {
    #[cfg(all(feature = "UIColor", feature = "block2"))]
    pub static UIConfigurationColorTransformerGrayscale: UIConfigurationColorTransformer;
}

extern "C" {
    #[cfg(all(feature = "UIColor", feature = "block2"))]
    pub static UIConfigurationColorTransformerPreferredTint: UIConfigurationColorTransformer;
}

extern "C" {
    #[cfg(all(feature = "UIColor", feature = "block2"))]
    pub static UIConfigurationColorTransformerMonochromeTint: UIConfigurationColorTransformer;
}
