//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIImageReaderConfiguration;

    unsafe impl ClassType for UIImageReaderConfiguration {
        type Super = NSObject;
        type Mutability = InteriorMutable;
    }
);

unsafe impl NSCopying for UIImageReaderConfiguration {}

unsafe impl NSObjectProtocol for UIImageReaderConfiguration {}

extern_methods!(
    unsafe impl UIImageReaderConfiguration {
        #[method(prefersHighDynamicRange)]
        pub unsafe fn prefersHighDynamicRange(&self) -> bool;

        #[method(setPrefersHighDynamicRange:)]
        pub unsafe fn setPrefersHighDynamicRange(&self, prefers_high_dynamic_range: bool);

        #[method(preparesImagesForDisplay)]
        pub unsafe fn preparesImagesForDisplay(&self) -> bool;

        #[method(setPreparesImagesForDisplay:)]
        pub unsafe fn setPreparesImagesForDisplay(&self, prepares_images_for_display: bool);

        #[method(preferredThumbnailSize)]
        pub unsafe fn preferredThumbnailSize(&self) -> CGSize;

        #[method(setPreferredThumbnailSize:)]
        pub unsafe fn setPreferredThumbnailSize(&self, preferred_thumbnail_size: CGSize);

        #[method(pixelsPerInch)]
        pub unsafe fn pixelsPerInch(&self) -> CGFloat;

        #[method(setPixelsPerInch:)]
        pub unsafe fn setPixelsPerInch(&self, pixels_per_inch: CGFloat);
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl UIImageReaderConfiguration {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new() -> Retained<Self>;
    }
);

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIImageReader;

    unsafe impl ClassType for UIImageReader {
        type Super = NSObject;
        type Mutability = InteriorMutable;
    }
);

unsafe impl NSObjectProtocol for UIImageReader {}

extern_methods!(
    unsafe impl UIImageReader {
        #[method_id(@__retain_semantics Other defaultReader)]
        pub unsafe fn defaultReader() -> Retained<UIImageReader>;

        #[method_id(@__retain_semantics Other readerWithConfiguration:)]
        pub unsafe fn readerWithConfiguration(
            configuration: &UIImageReaderConfiguration,
        ) -> Retained<Self>;

        #[method_id(@__retain_semantics Other configuration)]
        pub unsafe fn configuration(&self) -> Retained<UIImageReaderConfiguration>;

        #[cfg(feature = "UIImage")]
        #[method_id(@__retain_semantics Other imageWithContentsOfFileURL:)]
        pub unsafe fn imageWithContentsOfFileURL(&self, url: &NSURL) -> Retained<UIImage>;

        #[cfg(feature = "UIImage")]
        #[method_id(@__retain_semantics Other imageWithData:)]
        pub unsafe fn imageWithData(&self, data: &NSData) -> Retained<UIImage>;

        #[cfg(all(feature = "UIImage", feature = "block2"))]
        #[method(imageWithContentsOfFileURL:completion:)]
        pub unsafe fn imageWithContentsOfFileURL_completion(
            &self,
            url: &NSURL,
            completion: &block2::Block<dyn Fn(*mut UIImage)>,
        );

        #[cfg(all(feature = "UIImage", feature = "block2"))]
        #[method(imageWithData:completion:)]
        pub unsafe fn imageWithData_completion(
            &self,
            data: &NSData,
            completion: &block2::Block<dyn Fn(*mut UIImage)>,
        );
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl UIImageReader {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new() -> Retained<Self>;
    }
);
