//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_protocol!(
    pub unsafe trait UIUserActivityRestoring: NSObjectProtocol + IsMainThreadOnly {
        #[method(restoreUserActivityState:)]
        unsafe fn restoreUserActivityState(&self, user_activity: &NSUserActivity);
    }

    unsafe impl ProtocolType for dyn UIUserActivityRestoring {}
);
