//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIGraphicsRendererFormat;

    unsafe impl ClassType for UIGraphicsRendererFormat {
        type Super = NSObject;
        type Mutability = InteriorMutable;
    }
);

unsafe impl NSCopying for UIGraphicsRendererFormat {}

unsafe impl NSObjectProtocol for UIGraphicsRendererFormat {}

extern_methods!(
    unsafe impl UIGraphicsRendererFormat {
        #[deprecated]
        #[method_id(@__retain_semantics Other defaultFormat)]
        pub unsafe fn defaultFormat() -> Retained<Self>;

        #[method_id(@__retain_semantics Other preferredFormat)]
        pub unsafe fn preferredFormat() -> Retained<Self>;

        #[method(bounds)]
        pub unsafe fn bounds(&self) -> CGRect;
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl UIGraphicsRendererFormat {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new() -> Retained<Self>;
    }
);

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIGraphicsRendererContext;

    unsafe impl ClassType for UIGraphicsRendererContext {
        type Super = NSObject;
        type Mutability = InteriorMutable;
    }
);

unsafe impl NSObjectProtocol for UIGraphicsRendererContext {}

extern_methods!(
    unsafe impl UIGraphicsRendererContext {
        #[method_id(@__retain_semantics Other format)]
        pub unsafe fn format(&self) -> Retained<UIGraphicsRendererFormat>;

        #[method(fillRect:)]
        pub unsafe fn fillRect(&self, rect: CGRect);

        #[method(strokeRect:)]
        pub unsafe fn strokeRect(&self, rect: CGRect);

        #[method(clipToRect:)]
        pub unsafe fn clipToRect(&self, rect: CGRect);
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl UIGraphicsRendererContext {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new() -> Retained<Self>;
    }
);

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIGraphicsRenderer;

    unsafe impl ClassType for UIGraphicsRenderer {
        type Super = NSObject;
        type Mutability = InteriorMutable;
    }
);

unsafe impl NSObjectProtocol for UIGraphicsRenderer {}

extern_methods!(
    unsafe impl UIGraphicsRenderer {
        #[method_id(@__retain_semantics Init initWithBounds:)]
        pub unsafe fn initWithBounds(this: Allocated<Self>, bounds: CGRect) -> Retained<Self>;

        #[method_id(@__retain_semantics Init initWithBounds:format:)]
        pub unsafe fn initWithBounds_format(
            this: Allocated<Self>,
            bounds: CGRect,
            format: &UIGraphicsRendererFormat,
        ) -> Retained<Self>;

        #[method_id(@__retain_semantics Other format)]
        pub unsafe fn format(&self) -> Retained<UIGraphicsRendererFormat>;

        #[method(allowsImageOutput)]
        pub unsafe fn allowsImageOutput(&self) -> bool;
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl UIGraphicsRenderer {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new() -> Retained<Self>;
    }
);
