//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIPrintPaper;

    unsafe impl ClassType for UIPrintPaper {
        type Super = NSObject;
        type Mutability = MainThreadOnly;
    }
);

unsafe impl NSObjectProtocol for UIPrintPaper {}

extern_methods!(
    unsafe impl UIPrintPaper {
        #[method_id(@__retain_semantics Other bestPaperForPageSize:withPapersFromArray:)]
        pub unsafe fn bestPaperForPageSize_withPapersFromArray(
            content_size: CGSize,
            paper_list: &NSArray<UIPrintPaper>,
            mtm: MainThreadMarker,
        ) -> Retained<UIPrintPaper>;

        #[method(paperSize)]
        pub unsafe fn paperSize(&self) -> CGSize;

        #[method(printableRect)]
        pub unsafe fn printableRect(&self) -> CGRect;
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl UIPrintPaper {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;
    }
);

extern_methods!(
    /// Deprecated_Nonfunctional
    unsafe impl UIPrintPaper {
        #[method(printRect)]
        pub unsafe fn printRect(&self) -> CGRect;
    }
);
