//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;

use crate::*;

extern_protocol!(
    pub unsafe trait UIContentSizeCategoryAdjusting:
        NSObjectProtocol + IsMainThreadOnly
    {
        #[method(adjustsFontForContentSizeCategory)]
        unsafe fn adjustsFontForContentSizeCategory(&self) -> bool;

        #[method(setAdjustsFontForContentSizeCategory:)]
        unsafe fn setAdjustsFontForContentSizeCategory(
            &self,
            adjusts_font_for_content_size_category: bool,
        );
    }

    unsafe impl ProtocolType for dyn UIContentSizeCategoryAdjusting {}
);
