//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

// NS_TYPED_ENUM
pub type UIContentSizeCategory = NSString;

extern "C" {
    pub static UIContentSizeCategoryUnspecified: &'static UIContentSizeCategory;
}

extern "C" {
    pub static UIContentSizeCategoryExtraSmall: &'static UIContentSizeCategory;
}

extern "C" {
    pub static UIContentSizeCategorySmall: &'static UIContentSizeCategory;
}

extern "C" {
    pub static UIContentSizeCategoryMedium: &'static UIContentSizeCategory;
}

extern "C" {
    pub static UIContentSizeCategoryLarge: &'static UIContentSizeCategory;
}

extern "C" {
    pub static UIContentSizeCategoryExtraLarge: &'static UIContentSizeCategory;
}

extern "C" {
    pub static UIContentSizeCategoryExtraExtraLarge: &'static UIContentSizeCategory;
}

extern "C" {
    pub static UIContentSizeCategoryExtraExtraExtraLarge: &'static UIContentSizeCategory;
}

extern "C" {
    pub static UIContentSizeCategoryAccessibilityMedium: &'static UIContentSizeCategory;
}

extern "C" {
    pub static UIContentSizeCategoryAccessibilityLarge: &'static UIContentSizeCategory;
}

extern "C" {
    pub static UIContentSizeCategoryAccessibilityExtraLarge: &'static UIContentSizeCategory;
}

extern "C" {
    pub static UIContentSizeCategoryAccessibilityExtraExtraLarge: &'static UIContentSizeCategory;
}

extern "C" {
    pub static UIContentSizeCategoryAccessibilityExtraExtraExtraLarge:
        &'static UIContentSizeCategory;
}

extern "C" {
    pub static UIContentSizeCategoryDidChangeNotification: &'static NSNotificationName;
}

extern "C" {
    pub static UIContentSizeCategoryNewValueKey: &'static NSString;
}

extern "C" {
    pub fn UIContentSizeCategoryIsAccessibilityCategory(category: &UIContentSizeCategory) -> Bool;
}

extern "C" {
    pub fn UIContentSizeCategoryCompareToCategory(
        lhs: &UIContentSizeCategory,
        rhs: &UIContentSizeCategory,
    ) -> NSComparisonResult;
}
