//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;

use crate::*;

extern_methods!(
    /// UIActivityItemsConfiguration
    #[cfg(feature = "UIResponder")]
    unsafe impl UIResponder {
        #[cfg(feature = "UIActivityItemsConfigurationReading")]
        #[method_id(@__retain_semantics Other activityItemsConfiguration)]
        pub unsafe fn activityItemsConfiguration(
            &self,
        ) -> Option<Retained<ProtocolObject<dyn UIActivityItemsConfigurationReading>>>;

        #[cfg(feature = "UIActivityItemsConfigurationReading")]
        #[method(setActivityItemsConfiguration:)]
        pub unsafe fn setActivityItemsConfiguration(
            &self,
            activity_items_configuration: Option<
                &ProtocolObject<dyn UIActivityItemsConfigurationReading>,
            >,
        );
    }
);

#[cfg(all(
    feature = "UIActivityItemsConfigurationReading",
    feature = "UIResponder"
))]
unsafe impl UIActivityItemsConfigurationProviding for UIResponder {}
