//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

// NS_TYPED_ENUM
pub type UINibOptionsKey = NSString;

extern "C" {
    pub static UINibExternalObjects: &'static UINibOptionsKey;
}

extern "C" {
    pub static UINibProxiedObjectsKey: &'static NSString;
}
