//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;

use crate::*;

extern_methods!(
    /// AVAudioSession
    #[cfg(all(feature = "UIResponder", feature = "UIScene"))]
    unsafe impl UIScene {}
);
