//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UISceneActivationConditions;

    unsafe impl ClassType for UISceneActivationConditions {
        type Super = NSObject;
        type Mutability = MainThreadOnly;
    }
);

unsafe impl NSCoding for UISceneActivationConditions {}

unsafe impl NSObjectProtocol for UISceneActivationConditions {}

unsafe impl NSSecureCoding for UISceneActivationConditions {}

extern_methods!(
    unsafe impl UISceneActivationConditions {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics Init initWithCoder:)]
        pub unsafe fn initWithCoder(
            this: Allocated<Self>,
            a_decoder: &NSCoder,
        ) -> Option<Retained<Self>>;

        #[method_id(@__retain_semantics Other canActivateForTargetContentIdentifierPredicate)]
        pub unsafe fn canActivateForTargetContentIdentifierPredicate(
            &self,
        ) -> Retained<NSPredicate>;

        #[method(setCanActivateForTargetContentIdentifierPredicate:)]
        pub unsafe fn setCanActivateForTargetContentIdentifierPredicate(
            &self,
            can_activate_for_target_content_identifier_predicate: &NSPredicate,
        );

        #[method_id(@__retain_semantics Other prefersToActivateForTargetContentIdentifierPredicate)]
        pub unsafe fn prefersToActivateForTargetContentIdentifierPredicate(
            &self,
        ) -> Retained<NSPredicate>;

        #[method(setPrefersToActivateForTargetContentIdentifierPredicate:)]
        pub unsafe fn setPrefersToActivateForTargetContentIdentifierPredicate(
            &self,
            prefers_to_activate_for_target_content_identifier_predicate: &NSPredicate,
        );
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl UISceneActivationConditions {
        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;
    }
);

extern_category!(
    /// Category "UISceneActivationConditions" on [`NSUserActivity`].
    #[doc(alias = "UISceneActivationConditions")]
    pub unsafe trait NSUserActivityUISceneActivationConditions {
        #[method_id(@__retain_semantics Other targetContentIdentifier)]
        unsafe fn targetContentIdentifier(&self) -> Option<Retained<NSString>>;

        #[method(setTargetContentIdentifier:)]
        unsafe fn setTargetContentIdentifier(&self, target_content_identifier: Option<&NSString>);
    }

    unsafe impl NSUserActivityUISceneActivationConditions for NSUserActivity {}
);
