//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct UINotificationFeedbackType(pub NSInteger);
impl UINotificationFeedbackType {
    #[doc(alias = "UINotificationFeedbackTypeSuccess")]
    pub const Success: Self = Self(0);
    #[doc(alias = "UINotificationFeedbackTypeWarning")]
    pub const Warning: Self = Self(1);
    #[doc(alias = "UINotificationFeedbackTypeError")]
    pub const Error: Self = Self(2);
}

unsafe impl Encode for UINotificationFeedbackType {
    const ENCODING: Encoding = NSInteger::ENCODING;
}

unsafe impl RefEncode for UINotificationFeedbackType {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    #[cfg(feature = "UIFeedbackGenerator")]
    pub struct UINotificationFeedbackGenerator;

    #[cfg(feature = "UIFeedbackGenerator")]
    unsafe impl ClassType for UINotificationFeedbackGenerator {
        #[inherits(NSObject)]
        type Super = UIFeedbackGenerator;
        type Mutability = MainThreadOnly;
    }
);

#[cfg(feature = "UIFeedbackGenerator")]
unsafe impl NSObjectProtocol for UINotificationFeedbackGenerator {}

extern_methods!(
    #[cfg(feature = "UIFeedbackGenerator")]
    unsafe impl UINotificationFeedbackGenerator {
        #[method(notificationOccurred:)]
        pub unsafe fn notificationOccurred(&self, notification_type: UINotificationFeedbackType);

        #[method(notificationOccurred:atLocation:)]
        pub unsafe fn notificationOccurred_atLocation(
            &self,
            notification_type: UINotificationFeedbackType,
            location: CGPoint,
        );
    }
);

extern_methods!(
    /// Methods declared on superclass `UIFeedbackGenerator`
    #[cfg(feature = "UIFeedbackGenerator")]
    unsafe impl UINotificationFeedbackGenerator {
        #[cfg(all(feature = "UIResponder", feature = "UIView"))]
        #[method_id(@__retain_semantics Other feedbackGeneratorForView:)]
        pub unsafe fn feedbackGeneratorForView(view: &UIView) -> Retained<Self>;

        #[deprecated]
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    #[cfg(feature = "UIFeedbackGenerator")]
    unsafe impl UINotificationFeedbackGenerator {
        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;
    }
);
