//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIPrinterDestination;

    unsafe impl ClassType for UIPrinterDestination {
        type Super = NSObject;
        type Mutability = MainThreadOnly;
    }
);

unsafe impl NSCoding for UIPrinterDestination {}

unsafe impl NSObjectProtocol for UIPrinterDestination {}

unsafe impl NSSecureCoding for UIPrinterDestination {}

extern_methods!(
    unsafe impl UIPrinterDestination {
        #[method_id(@__retain_semantics Init initWithURL:)]
        pub unsafe fn initWithURL(this: Allocated<Self>, url: &NSURL) -> Retained<Self>;

        #[method_id(@__retain_semantics Other URL)]
        pub unsafe fn URL(&self) -> Retained<NSURL>;

        #[method(setURL:)]
        pub unsafe fn setURL(&self, url: &NSURL);

        #[method_id(@__retain_semantics Other displayName)]
        pub unsafe fn displayName(&self) -> Option<Retained<NSString>>;

        #[method(setDisplayName:)]
        pub unsafe fn setDisplayName(&self, display_name: Option<&NSString>);

        #[method_id(@__retain_semantics Other txtRecord)]
        pub unsafe fn txtRecord(&self) -> Option<Retained<NSData>>;

        #[method(setTxtRecord:)]
        pub unsafe fn setTxtRecord(&self, txt_record: Option<&NSData>);
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl UIPrinterDestination {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;
    }
);

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIPrintServiceExtension;

    unsafe impl ClassType for UIPrintServiceExtension {
        type Super = NSObject;
        type Mutability = MainThreadOnly;
    }
);

unsafe impl NSObjectProtocol for UIPrintServiceExtension {}

extern_methods!(
    unsafe impl UIPrintServiceExtension {
        #[cfg(feature = "UIPrintInfo")]
        #[method_id(@__retain_semantics Other printerDestinationsForPrintInfo:)]
        pub unsafe fn printerDestinationsForPrintInfo(
            &self,
            print_info: &UIPrintInfo,
        ) -> Retained<NSArray<UIPrinterDestination>>;
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl UIPrintServiceExtension {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;
    }
);
