//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct UITextDropAction(pub NSUInteger);
impl UITextDropAction {
    #[doc(alias = "UITextDropActionInsert")]
    pub const Insert: Self = Self(0);
    #[doc(alias = "UITextDropActionReplaceSelection")]
    pub const ReplaceSelection: Self = Self(1);
    #[doc(alias = "UITextDropActionReplaceAll")]
    pub const ReplaceAll: Self = Self(2);
}

unsafe impl Encode for UITextDropAction {
    const ENCODING: Encoding = NSUInteger::ENCODING;
}

unsafe impl RefEncode for UITextDropAction {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct UITextDropProgressMode(pub NSUInteger);
impl UITextDropProgressMode {
    #[doc(alias = "UITextDropProgressModeSystem")]
    pub const System: Self = Self(0);
    #[doc(alias = "UITextDropProgressModeCustom")]
    pub const Custom: Self = Self(1);
}

unsafe impl Encode for UITextDropProgressMode {
    const ENCODING: Encoding = NSUInteger::ENCODING;
}

unsafe impl RefEncode for UITextDropProgressMode {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct UITextDropPerformer(pub NSUInteger);
impl UITextDropPerformer {
    #[doc(alias = "UITextDropPerformerView")]
    pub const View: Self = Self(0);
    #[doc(alias = "UITextDropPerformerDelegate")]
    pub const Delegate: Self = Self(1);
}

unsafe impl Encode for UITextDropPerformer {
    const ENCODING: Encoding = NSUInteger::ENCODING;
}

unsafe impl RefEncode for UITextDropPerformer {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    #[cfg(feature = "UIDropInteraction")]
    pub struct UITextDropProposal;

    #[cfg(feature = "UIDropInteraction")]
    unsafe impl ClassType for UITextDropProposal {
        #[inherits(NSObject)]
        type Super = UIDropProposal;
        type Mutability = MainThreadOnly;
    }
);

#[cfg(feature = "UIDropInteraction")]
unsafe impl NSCopying for UITextDropProposal {}

#[cfg(feature = "UIDropInteraction")]
unsafe impl NSObjectProtocol for UITextDropProposal {}

extern_methods!(
    #[cfg(feature = "UIDropInteraction")]
    unsafe impl UITextDropProposal {
        #[method(dropAction)]
        pub unsafe fn dropAction(&self) -> UITextDropAction;

        #[method(setDropAction:)]
        pub unsafe fn setDropAction(&self, drop_action: UITextDropAction);

        #[method(dropProgressMode)]
        pub unsafe fn dropProgressMode(&self) -> UITextDropProgressMode;

        #[method(setDropProgressMode:)]
        pub unsafe fn setDropProgressMode(&self, drop_progress_mode: UITextDropProgressMode);

        #[method(useFastSameViewOperations)]
        pub unsafe fn useFastSameViewOperations(&self) -> bool;

        #[method(setUseFastSameViewOperations:)]
        pub unsafe fn setUseFastSameViewOperations(&self, use_fast_same_view_operations: bool);

        #[method(dropPerformer)]
        pub unsafe fn dropPerformer(&self) -> UITextDropPerformer;

        #[method(setDropPerformer:)]
        pub unsafe fn setDropPerformer(&self, drop_performer: UITextDropPerformer);
    }
);

extern_methods!(
    /// Methods declared on superclass `UIDropProposal`
    #[cfg(feature = "UIDropInteraction")]
    unsafe impl UITextDropProposal {
        #[method_id(@__retain_semantics Init initWithDropOperation:)]
        pub unsafe fn initWithDropOperation(
            this: Allocated<Self>,
            operation: UIDropOperation,
        ) -> Retained<Self>;

        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;
    }
);
