//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIPasteConfiguration;

    unsafe impl ClassType for UIPasteConfiguration {
        type Super = NSObject;
        type Mutability = MainThreadOnly;
    }
);

unsafe impl NSCoding for UIPasteConfiguration {}

unsafe impl NSCopying for UIPasteConfiguration {}

unsafe impl NSObjectProtocol for UIPasteConfiguration {}

unsafe impl NSSecureCoding for UIPasteConfiguration {}

extern_methods!(
    unsafe impl UIPasteConfiguration {
        #[method_id(@__retain_semantics Other acceptableTypeIdentifiers)]
        pub unsafe fn acceptableTypeIdentifiers(&self) -> Retained<NSArray<NSString>>;

        #[method(setAcceptableTypeIdentifiers:)]
        pub unsafe fn setAcceptableTypeIdentifiers(
            &self,
            acceptable_type_identifiers: &NSArray<NSString>,
        );

        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics Init initWithAcceptableTypeIdentifiers:)]
        pub unsafe fn initWithAcceptableTypeIdentifiers(
            this: Allocated<Self>,
            acceptable_type_identifiers: &NSArray<NSString>,
        ) -> Retained<Self>;

        #[method(addAcceptableTypeIdentifiers:)]
        pub unsafe fn addAcceptableTypeIdentifiers(
            &self,
            acceptable_type_identifiers: &NSArray<NSString>,
        );

        #[method_id(@__retain_semantics Init initWithTypeIdentifiersForAcceptingClass:)]
        pub unsafe fn initWithTypeIdentifiersForAcceptingClass(
            this: Allocated<Self>,
            a_class: &AnyClass,
        ) -> Retained<Self>;

        #[method(addTypeIdentifiersForAcceptingClass:)]
        pub unsafe fn addTypeIdentifiersForAcceptingClass(&self, a_class: &AnyClass);
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    unsafe impl UIPasteConfiguration {
        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;
    }
);
