//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UIStatusBarManager;

    unsafe impl ClassType for UIStatusBarManager {
        type Super = NSObject;
        type Mutability = MainThreadOnly;
    }
);

unsafe impl NSObjectProtocol for UIStatusBarManager {}

extern_methods!(
    unsafe impl UIStatusBarManager {
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;

        #[cfg(feature = "UIApplication")]
        #[method(statusBarStyle)]
        pub unsafe fn statusBarStyle(&self) -> UIStatusBarStyle;

        #[method(isStatusBarHidden)]
        pub unsafe fn isStatusBarHidden(&self) -> bool;

        #[method(statusBarFrame)]
        pub unsafe fn statusBarFrame(&self) -> CGRect;
    }
);

extern_methods!(
    /// StatusBarManager
    #[cfg(all(
        feature = "UIResponder",
        feature = "UIScene",
        feature = "UIWindowScene"
    ))]
    unsafe impl UIWindowScene {
        #[method_id(@__retain_semantics Other statusBarManager)]
        pub unsafe fn statusBarManager(&self) -> Option<Retained<UIStatusBarManager>>;
    }
);
