//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    #[cfg(feature = "UIFeedbackGenerator")]
    pub struct UICanvasFeedbackGenerator;

    #[cfg(feature = "UIFeedbackGenerator")]
    unsafe impl ClassType for UICanvasFeedbackGenerator {
        #[inherits(NSObject)]
        type Super = UIFeedbackGenerator;
        type Mutability = MainThreadOnly;
    }
);

#[cfg(feature = "UIFeedbackGenerator")]
unsafe impl NSObjectProtocol for UICanvasFeedbackGenerator {}

extern_methods!(
    #[cfg(feature = "UIFeedbackGenerator")]
    unsafe impl UICanvasFeedbackGenerator {
        #[method(alignmentOccurredAtLocation:)]
        pub unsafe fn alignmentOccurredAtLocation(&self, location: CGPoint);

        #[method(pathCompletedAtLocation:)]
        pub unsafe fn pathCompletedAtLocation(&self, location: CGPoint);
    }
);

extern_methods!(
    /// Methods declared on superclass `UIFeedbackGenerator`
    #[cfg(feature = "UIFeedbackGenerator")]
    unsafe impl UICanvasFeedbackGenerator {
        #[cfg(all(feature = "UIResponder", feature = "UIView"))]
        #[method_id(@__retain_semantics Other feedbackGeneratorForView:)]
        pub unsafe fn feedbackGeneratorForView(view: &UIView) -> Retained<Self>;

        #[deprecated]
        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;
    }
);

extern_methods!(
    /// Methods declared on superclass `NSObject`
    #[cfg(feature = "UIFeedbackGenerator")]
    unsafe impl UICanvasFeedbackGenerator {
        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new(mtm: MainThreadMarker) -> Retained<Self>;
    }
);
