//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;

use crate::*;

// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct UILetterformAwareSizingRule(pub NSInteger);
impl UILetterformAwareSizingRule {
    #[doc(alias = "UILetterformAwareSizingRuleTypographic")]
    pub const Typographic: Self = Self(0);
    #[doc(alias = "UILetterformAwareSizingRuleOversize")]
    pub const Oversize: Self = Self(1);
}

unsafe impl Encode for UILetterformAwareSizingRule {
    const ENCODING: Encoding = NSInteger::ENCODING;
}

unsafe impl RefEncode for UILetterformAwareSizingRule {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

extern_protocol!(
    pub unsafe trait UILetterformAwareAdjusting:
        NSObjectProtocol + IsMainThreadOnly
    {
        #[method(sizingRule)]
        unsafe fn sizingRule(&self) -> UILetterformAwareSizingRule;

        #[method(setSizingRule:)]
        unsafe fn setSizingRule(&self, sizing_rule: UILetterformAwareSizingRule);
    }

    unsafe impl ProtocolType for dyn UILetterformAwareAdjusting {}
);
