//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern "C" {
    pub static UIPrintErrorDomain: &'static NSErrorDomain;
}

// NS_ERROR_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct UIPrintErrorCode(pub NSInteger);
impl UIPrintErrorCode {
    pub const UIPrintingNotAvailableError: Self = Self(1);
    pub const UIPrintNoContentError: Self = Self(2);
    pub const UIPrintUnknownImageFormatError: Self = Self(3);
    pub const UIPrintJobFailedError: Self = Self(4);
}

unsafe impl Encode for UIPrintErrorCode {
    const ENCODING: Encoding = NSInteger::ENCODING;
}

unsafe impl RefEncode for UIPrintErrorCode {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}
