//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use objc2::__framework_prelude::*;
use objc2_foundation::*;

use crate::*;

extern_class!(
    #[derive(Debug, PartialEq, Eq, Hash)]
    pub struct UISceneSessionActivationRequest;

    unsafe impl ClassType for UISceneSessionActivationRequest {
        type Super = NSObject;
        type Mutability = InteriorMutable;
    }
);

unsafe impl NSCopying for UISceneSessionActivationRequest {}

unsafe impl NSObjectProtocol for UISceneSessionActivationRequest {}

extern_methods!(
    unsafe impl UISceneSessionActivationRequest {
        #[cfg(feature = "UISceneDefinitions")]
        #[method_id(@__retain_semantics Other role)]
        pub unsafe fn role(&self) -> Retained<UISceneSessionRole>;

        #[cfg(feature = "UISceneSession")]
        #[method_id(@__retain_semantics Other session)]
        pub unsafe fn session(&self, mtm: MainThreadMarker) -> Option<Retained<UISceneSession>>;

        #[method_id(@__retain_semantics Other userActivity)]
        pub unsafe fn userActivity(&self) -> Option<Retained<NSUserActivity>>;

        #[method(setUserActivity:)]
        pub unsafe fn setUserActivity(&self, user_activity: Option<&NSUserActivity>);

        #[cfg(feature = "UISceneOptions")]
        #[method_id(@__retain_semantics Other options)]
        pub unsafe fn options(
            &self,
            mtm: MainThreadMarker,
        ) -> Option<Retained<UISceneActivationRequestOptions>>;

        #[cfg(feature = "UISceneOptions")]
        #[method(setOptions:)]
        pub unsafe fn setOptions(&self, options: Option<&UISceneActivationRequestOptions>);

        #[method_id(@__retain_semantics Other request)]
        pub unsafe fn request() -> Retained<Self>;

        #[cfg(feature = "UISceneDefinitions")]
        #[method_id(@__retain_semantics Other requestWithRole:)]
        pub unsafe fn requestWithRole(role: &UISceneSessionRole) -> Retained<Self>;

        #[cfg(feature = "UISceneSession")]
        #[method_id(@__retain_semantics Other requestWithSession:)]
        pub unsafe fn requestWithSession(session: &UISceneSession) -> Retained<Self>;

        #[method_id(@__retain_semantics Init init)]
        pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>;

        #[method_id(@__retain_semantics New new)]
        pub unsafe fn new() -> Retained<Self>;
    }
);
